<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');

use Joomla\CMS\MVC\Controller\FormController;
use Joomla\CMS\Factory;
use Joomla\CMS\Table\Table;
use Joomla\CMS\Session\Session;
use Joomla\CMS\Language\Text;
use Joomla\String\StringHelper;


class DatafeedsControllerDatafeed extends FormController
{
    public function __construct(array $config = array(), MVCFactoryInterface $factory = null, $app = null, $input = null)
    {
                                parent::__construct($config, $factory, $app, $input);

        $this->registerTask('add', 'edit');
        $this->registerTask('import', 'edit');
        $this->registerTask('copy', 'edit');
        $this->registerTask('apply', 'save');
        $this->feed_velden = [
        'field1','field2','field3','field4','field5','field6','field7','field8','field9',
        'field10','field11','field12','field13','field14','field15','field16','field17','field18','field19',
        'title','url','image','price'];#,'unid');
    }

    public function edit($key = null, $urlVar = null)
    {
        $input = Factory::getApplication()->input;
        $option = $input->get('option');

        $lists = [];
        if ($input->get('task') == 'edit' ||$input->get('task') == 'copy') {
            $cid    = $input->get('cid', -1);
        } else {
            $cid    = 0;
        }
        if (is_array($cid)) {
            $cid=(int)$cid[0];
        }

        if ($cid> -1) {
            $row =Table::getInstance('Datafeeds', 'Table');
            $row->load($cid);
            if ($cid > 0 && ! $row->feed_id) {
                $link = 'index.php?option=com_datafeeds&c=datafeeds';
                $this->setRedirect($link, 'Unknown feed') ;
                return;
            }


            if ($input->get('task') == 'copy') {
                $row->feed_id=0;
                $row->feed=$row->feed.":copy";
                $row->published=0;
            }
            $opties=[];
            $row->has_data=false;
            foreach ($this->feed_velden as $v) {
                if ($row->$v) {
                    $row->has_data=true;
                    $opties[$row->$v]=  "->".$row->$v;
                }
            }
            $example_raw_array=json_decode($row->example_raw, true);
            if (is_array($example_raw_array)) {
                $row->has_data=true;
                foreach ($example_raw_array as $k => $v) {
                    $k=StringHelper::strtolower($k);
                    $opties[$k] = $k;
                }
            }
            ksort($opties);
            $row->published=((bool)$row->published)?1:0;
            require_once(JPATH_COMPONENT.'/views/datafeed.php');
            DatafeedsViewListFeed::display($row, $opties);
        } else {
            $link = 'index.php?option=com_datafeeds&c=datafeeds';
            $this->setRedirect($link, 'No feed selected') ;
        }
    }

    public function save($key = null, $urlVar = null)
    {
        global $feed_opts;
        $msg_type='message';
        Session::checkToken() or jexit(Text::_('JINVALID_TOKEN'));
        $input = Factory::getApplication()->input;
        $post = $input->get('jform', array(), 'ARRAY');
        $row =Table::getInstance('datafeeds', 'Table');
        $this->setRedirect('index.php?option=com_datafeeds&c=datafeeds');
        if ($post['feed_id']) {
            $row->load($post['feed_id']);
        }


        if (!$row->bind($post)) {
            Factory::getApplication()->enqueueMessage("Bind failed: " . $row->getError(), 'error');
            return false;
        }

        $task = $input->get('task');
        if ($task == 'apply') {
            require_once(JPATH_COMPONENT.'/cron/generic_cb.php');
            if ($row->class=='AutoCSV' && $class=$row->getClass()) {
                $row->class=$class;
            }
            if (! function_exists($row->callback)) {
                $callback=function (&$item) {
                    return;
                };
            } else {
                $callback=$row->callback;
            }
            $feed_opts['feed']=preg_replace("#:.*#", "", $row->feed);
            $feed_opts['field0']=$row->field0; # == field0
            foreach ($this->feed_velden as $v) {
                $feed_opts[$v]=$row->$v;
            }
            if (! defined('MAGPIE_DIR')) {
                     define('MAGPIE_DIR', JPATH_COMPONENT.'/cron/');
            }
            if (! function_exists('bare_fetch')) {
                require_once(JPATH_COMPONENT.'/cron/xml_fetch.php');
            }

            $rss = bare_fetch(
                $row->feed_url,
                $row->class,
                $callback
            );

            $status=is_bool($rss)?$rss:$rss->status;
            if ($status) {
                $row->example=  json_encode($rss->examples);
                $rss->examples=[];#memory usage reduction
                $row->example_raw=  json_encode($rss->example_raw);
                $rss->example_raw=[];
            } else {
                $row->example=json_encode(["Failed to fetch feed"]);
                if ($row->example_raw) {
                    $example_raw_array=json_decode($row->example_raw, true);
                    if (is_array($example_raw_array)) {
                        foreach ($example_raw_array as $k => $v) {
                            $example_raw_array[$k]='Failed to fetch feed';
                        }
                        if ($rss->error) {
                            $example_raw_array=array_merge(['error'=>'<strong>'.$rss->error.'</strong>'], $example_raw_array);
                        }
                        $row->example_raw=  json_encode($example_raw_array);
                    }
                }
            }
        }

        if (!$row->check()) {
            Factory::getApplication()->enqueueMessage("Check failed: " . $row->getError(), 'error');
            return false;
        }
        if (!$row->store()) {
            Factory::getApplication()->enqueueMessage("Store failed: " . $row->getError(), 'error');
            return false;
        }
        if (!$row->checkin()) {
            Factory::getApplication()->enqueueMessage("Checkin failed: " . $row->getError(), 'error');
            return false;
        }
        switch ($task) {
            case 'apply':
                $link = 'index.php?option=com_datafeeds&c=datafeed&task=edit&cid='.$row->feed_id ;
                if ($status) {
                    $msg=Text::_('ITEM_DATA_FETCHED') ;
                } else {
                    $msg=Text::_('FAILED_TO_PARSE_FEED') ;
                    if ($error=$rss->geterror()) {
                        $msg .=" ({$error})";
                    }
                    $msg_type='error';
                }
                break;
            case 'save':
            default:
                $link = 'index.php?option=com_datafeeds&c=datafeeds';
                $msg=Text::_('ITEM_SAVED') ;
                break;
        }
        $this->setRedirect($link, $msg, $msg_type) ;
    }

    public function cancel($key = null)
    {
        $this->setRedirect('index.php?option=com_datafeeds&c=datafeeds');
        $input = Factory::getApplication()->input;
        $post = $input->get('jform', array(), 'ARRAY');
        $row    =Table::getInstance('datafeeds', 'Table');
        $row->bind($post);
        $row->checkin();
    }
}
