<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');
 
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Application\ApplicationHelper;

class DatafeedsControllerEdit extends JControllerLegacy
{
    public function __construct($config = [])
    {
        parent::__construct($config);
        $this->registerTask('edit', 'display');
        $this->registerTask('cancel', 'display');
        $this->registerTask('apply', 'save');
        $path       = JPATH_ADMINISTRATOR . '/components/com_datafeeds/cron';
        $this->file = $path . '/feeds.php';
    }
    /**
     * constructor (registers additional tasks to methods)
     * @return void
     */
    /**
     * Method to display the view
     *
     * @access  public
     */
    public function display($cachable = false, $urlparams = false)
    {
        $app    = Factory::getApplication();
        $input = $app->input;
        $option = $input->get('option');
        $client = ApplicationHelper::getClientInfo($input->get('client', 0, 'int'));
        $content = file_get_contents($this->file);
        $task    = $input->get('task');
        $task    = $task ? $task : 'display';
        if ($content !== false) {
            require_once(JPATH_COMPONENT_ADMINISTRATOR . '/views/edit.php');
            DatafeedssViewedit::$task($content, $option, $client);
        } //$content !== false
        else {
            $msg = Text::sprintf('COM_DATAFEEDS_OPERATION_FAILED Could not open', $this->file);
            $app->redirect('index.php?option=' . $option . '&client=' . $client->id, $msg);
        }
    }
    public function save()
    {
        $app = Factory::getApplication();
        $user = Factory::getUser();
        $input = $app->input;
        $option = $input->get('option');
        if (!$user->get('isRoot')) {
            $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . Text::sprintf('Must be super administrator', $this->file));
            $app->redirect('index.php?option=' . $option . '&c=edit');
        } //!$user->get('isRoot')
        JSession::checkToken() or jexit(Text::_('JINVALID_TOKEN'));
        $client = ApplicationHelper::getClientInfo($input->get('client', 0, 'int'));
        $filecontent = $input->get('jform', [], 'array')['filecontent'];
        if (!$filecontent) {
            $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . Text::_('Content empty.'));
            $app->redirect('index.php?option=' . $option . '&client=' . $client->id);
        } //!$filecontent
        // Set FTP credentials, if given
        $return = File::write($this->file, $filecontent);
        if ($return) {
            $task = $input->get('task');
            switch ($task) {
                case 'apply':
                    $app->enqueueMessage(Text::_('feeds.php saved'));
                    $app->redirect('index.php?option=' . $option . '&task=edit&c=edit');
                    break;
                case 'save':
                default:
                    $app->enqueueMessage(Text::_('feeds.php saved'));
                    $app->redirect('index.php?option=' . $option . '&c=edit');
                    break;
            } //$task
        } //$return
        else {
            $app->enqueueMessage(Text::_('COM_DATAFEEDS_OPERATION_FAILED') . ': ' . Text::sprintf('Failed to open file for writing.', $this->file), 'error');
            $app->redirect('index.php?option=' . $option . '&c=edit');
        }
    }
}
