<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

 \defined('_JEXEC') or die('Restricted access');
 
use Joomla\CMS\Factory;
use Joomla\CMS\Language\Text;

if (file_exists(dirname(__FILE__).'/xhelpers.php')) {
    include_once dirname(__FILE__).'/xhelpers.php';
}

if ( !defined('DF_LEVELSTART')) {
    define('DF_LEVELSTART',1);
}
if ( !defined('DF_LEVELEND')) {
    define('DF_LEVELEND',9);
}

function pricequery($min, $max)
{
    if (!$min && !$max) {
        return '';
    } // fail safe
    if ($max && $min) {
        if ($max < $min) {
            $x = $max;
            $max = $min;
            $min = $x;
        }

        return " (`price` >= $min AND `price` <= $max) ";
    }
    if ($max) {
        return " (`price` < $max )";
    }

    return "( `price` >= $min )";
}
    //$query=$low_value?"($max_query AND $min_query)":$max_query;
function where_price($field, $value, $sequence)
{
    $first = 0;
    $tot_prijs = Text::_('TOTPRIJS');
    $tot_munt = Text::_('TOTMUNT');
    $value = trim(str_replace([$tot_prijs, $tot_munt], ['', ''], $value));
    foreach (explode(',', $sequence) as $r) {
        list($a, $b) = explode('-', $r);
        $first = $first ? $first : $b;
        if ($value <= $a) {
            continue;
        }
        $floor = "floor($b*ceil($field/$b))";
        if ($a) {
            $query = "IF ( $field > $a,$floor,0)";
        } else {
            $query = $floor;
        }
    }
    $query = "IF ($field=0,$first,$query)";

    return [$value, $query];
}


function make_qQuery($q, &$db)
{
    static $queries = [];
    $query = [];
    $input = Factory::getApplication()->input;

    $a = $input->get('areas', '', 'string');
    $exact = $input->get('exact', 0, 'int');
    if ('' == $a && defined('DF_AREAS')) {
        $a = DF_AREAS;
    }

    if ('' == $a) {
	    $a = [
		     '0' , '1' , '2' , '3' , '4' , '5' , '6' , '7' , '8' , '9' , 
		     '10' , '11' , '12' , '13' , '14' , '15' , '16' , '17' , '18' , '19' , 
		     't' , 'd' , 'f' 
	    ];
    }
    #wildcard searching is expensive therefor the description in not searched by default
    #you can enable it by using
    #define('DF_AREAS','0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,t,d,f);
    #the same to limit the fields searched to improve performance
    
    if (is_string($a)) {
        $a = explode(',', $a);
    }
    $areas=array_flip($a);

    $md5 = md5(join('', $areas).$q.$exact);
    if (isset($queries[$md5])) {
        return $queries[$md5];
    }
    foreach (explode('>', $q) as $v) {
        $ors = [];
        $ands = [];
        $nots = [];
        preg_match_all("/([-\*])?(\"[^\"]*\"|'[^']*'|\\S+)/", $v, $terms, PREG_SET_ORDER);

        foreach ($terms as $term) {
            $word = trim($term[2], '\'"');
            if ($exact) {
                $word = ' = '.$db->Quote($db->Escape($word, true), false);
            } else {
                $word = ' LIKE '.$db->Quote('%'.$db->Escape($word, true).'%', false);
            }
            $wheres2 = [];
            for ($i = 0; $i <= 19; ++$i) {
                if (isset($areas[$i])) {
                    $wheres2[] = 'field'.$i.$word;
                }
            }
            if (isset($areas['f'])) {
                $wheres2[] = '`feed` '.$word;
            }

            if (isset($areas['d'])) {
                $wheres2[] = '`description` '.$word;
            }
            if (isset($areas['t'])) {
                $wheres2[] = '`title` '.$word;
            }
            if (count($wheres2)) {
                $this_term = implode(' OR ', $wheres2);
                if ('-' == $term[1]) {
                    $this_term = ' NOT ( '.$this_term.')';
                    $nots[] = $this_term;
                } elseif ('*' == $term[1]) {
                    $ands[] = $this_term;
                } else {
                    $ors[] = $this_term;
                }
            }
        }
        if (count($ors)) {
            if (count($ands)) {
                $ands[] = '('.implode(') OR (', $ors).')'."\n";
            } else {
                $ands[] = '('.implode(') AND (', $ors).')'."\n";
            }
        }
        if (count($nots)) {
            $ands[] = '('.implode(') AND (', $nots).')'."\n";
        }
        if (count($ands)) {
            $query[] = '('.implode(') AND (', $ands).')'."\n";
        }
    }
    if (count($query)) {
        $query_string = '(('.implode(') OR (', $query).'))'."\n";
    } else {
        $query_string = '1';
    }
    $queries[$md5] = $query_string;

    return $query_string;
}


if (!function_exists('df_title')) {
    function df_title($f = '', $d = [])
    {
        return $f.'|'.join(' - ', $d);
    }
}
if (!function_exists('df_alt')) {
    function df_alt($str = '')
    {
        return 'Product';
    }
}

if (!function_exists('df_select')) {
    function df_select($f, $s, $not = false)
    {
 // weird order historical
        static $db = false;
        if (!$db) {
            $db = Factory::getContainer()->get('DatabaseDriver');
        }

        $t = $db->q($s);
        $f = $db->qn($f);
        if (false === $not) {
            return  "$f = $t";
        } else {
            return  "$f not in ('',$t)";
        }
    }
}
