<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

\defined('_JEXEC') or die('Restricted access');


use Joomla\CMS\MVC\Model\ListModel;
use Joomla\CMS\Factory;
use Joomla\CMS\Router\Route;
use Joomla\Registry\Registry;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Pagination\Pagination;
use Joomla\String\StringHelper;



class DatafeedsModelItems extends ListModel

{
    private $datacount = null;
    private $params = null;
    private $dataitems = null;
    private $where = [1];
    private $aantal = 20;
    private $where_level = 0;
    private $limitstart = 0;

    private $select = [];
    private $title = [];
    private $fields = [];
    private $singleitem = false;
    private $basequery = [];
    private $db;
    private $pagination;
    private $protection = 0;
    private $multipagina = 0;
    private $input;
    private $menu_id;
    private $maxq = 0;
    private $chain = 0; // is it a nice path /term/term2/term3


    private $group_select = '';
    private $volgorde = 'DESC';
    private $sorteer = '`items_id`';

    private function getInt($name, $default = 0)
    {
        return $this->input->get($name, $default, 'int');
    }
    private function getVar($name, $default = null)
    {
        return $this->input->get($name, $default, 'string');
    }
    private function setVar($name, $value = null)
    {
        return $this->input->set($name, $value);
    }
    public function __construct($config = array(), MVCFactoryInterface $factory = null)
    {
        parent::__construct($config, $factory);
        $app = Factory::getApplication();
        $this->db = $this->getDatabase();
        $this->input = $app->input;
        $menus = $app->getMenu();
        $menu = $menus->getActive();
        $locale = Factory::getLanguage()->getLocale()[0];
        setlocale(LC_MONETARY, $locale);


        if ($menu && 'com_datafeeds' == $menu->component) {
            $this->menu_id = $menu->id;
            $this->params = $app->getParams();

            $this->name = $menu->name ?? '';
        } else {  // ajax style request
            $_itemid = $this->getInt('Itemid', 0);
            if ($_itemid) {
                $menus->setActive($_itemid);
                //trac 101 $menus->setDefault($_itemid); # is deze nog nodig ?
                $menu = $menus->getActive();
                $this->menu_id = $menu->id;

                $this->params = $menu->getParams();
            } else {
                $this->params = new Registry('');
            }
        }
        $this->params = DatafeedsModelItems::fixParamNames($this->params);
        if ('com_datafeeds' != $menu->component || isset($config['ignore_request'])) {
            return;
        }

        $this->basequery = [
            'option' => 'com_datafeeds',
            'view' => 'items',
            'Itemid' => $this->menu_id
        ];
        $this->select['f'] = $this->getVar('qf');
        $this->select['t'] = $this->getVar('qt');
        $this->select['q'] = $this->getVar('q');
        $this->select['id'] = $this->getInt('id');
        if ($this->select['f']) {
            $this->title['f'] = $this->select['f'];
        }
        if ($this->select['t']) {
            $this->title['t'] = $this->select['t'];
        }
        if ($this->select['q']) {
            $this->title['q'] = $this->select['q'];
        }
        for ($v = DF_LEVELSTART; $v <= DF_LEVELEND; ++$v) {
            if ($this->select[$v] = $this->getVar('q' . $v)) {
                if (false !== $this->chain && ($v == $this->chain + 1)) {
                    $this->chain = $v;
                } else {
                    $this->chain = false;
                }
                $this->maxq = $v;
                if (is_array($this->select[$v])) {
                    $this->title[$v] = join(' ', $this->select[$v]);
                } else {
                    $this->title[$v] = $this->select[$v];
                }
            }
        }
        if (($m = $this->getVar('mm'))) {
            $x = explode(',', $m);
            $this->setVar('min', $x[0]);
            $this->setVar('max', $x[1]);
        }

        if (!$this->select['t'] && !$this->select['id']) {
            $this->select['min'] = $this->getVar('min');
            $this->select['max'] = $this->getVar('max');
        } else {
            unset($this->select['min']);
            unset($this->select['max']);
        }

        if (($m = $this->getVar('so'))) {
            $x = explode(',', $m);
            if (count($x)) {
                $this->setVar('sort', $x[0]);
                $this->setVar('order', $x[1]);
            }
        }

        $this->limitstart = $this->getInt('limitstart', 0);
        if ($num = $this->getInt('num', 0)) {
            if ($num > 0 && $num < 100) {
                $this->params->set('aantal0', $num);
                $this->params->set('aantal1', $num);
            }
        }
    }
    # redirects if there are no items
    #
    public function checkEmpty()
    {
        # seperated from __contructor
        $datacount = $this->datacount;
        $slug_level = -1;
        #Factory::getApplication()->enqueueMessage( "{$this->limitstart} $datacount;");

        //  onderstaande zorgt voor redirects als er niets gevonden wordt
        if (($datacount < 1) || ($this->limitstart >= $datacount)) {
            if (($datacount > 0) && ($this->limitstart >= $datacount)) {
                $slug_level = 9;
                $this->limitstart = 0;
                $this->setVar('limitstart', 0);
            } elseif ($this->select['q']) {
                $this->select['q'] = '';
                $this->setVar('q', '');
                $slug_level = 9;
            } elseif ($this->select['id']) {
                $this->select['id'] = '';
                $this->setVar('id', '');
                $slug_level = 9;
            } elseif ($this->select['f']) {
                $this->setVar('qf', '');
                $this->select['f'] = '';
                $slug_level = 9;
            } elseif ($this->select['t']) {
                $this->setVar('qt', '');
                $this->select['t'] = '';
                $slug_level = 9;
            } elseif ($this->select['min']) {
                $this->setVar('min');
                $this->setVar('max');
                $this->setVar('mm');
                unset($this->select['min']);
                unset($this->select['max']);
                $slug_level = 9;
            } else {
                for ($v = DF_LEVELEND; $v >= DF_LEVELSTART; --$v) {
                    if ($this->select[$v]) {
                        $this->setVar('q' . $v, '');
                        $this->select[$v] = '';
                        $slug_level = $v - ($this->select['t'] ? 0 : 1); // in nieuwe structuur komt 't' niet meer voor
                        $v = -1;
                    }
                }
            }

            if ($slug_level > -1) {
                $onempty = $this->params->get('onempty', 'redirect');
                $textonempty = $this->params->get('textonempty', '');
                switch ($onempty) {
                    case 'resp404':
                        header($_SERVER['SERVER_PROTOCOL'] . ' 404 Not Found');
                        header('Status: 404 Not Found');
                        //fall thru intended
                    case 'resp200':
                        if (!defined('DF_DID_REQUERY')) {
                            define('DF_DID_REQUERY', 1);
                        }
                        $document = Factory::getDocument();
                        $document->setMetaData('robots', 'NOinDEX,follow');

                        if (empty($this->select['t'])) {
                            if ($textonempty) {
                                $this->params->set('textonempty', '');
                                if ('tmpl' == $this->getVar('format')) {
                                    echo "<!--$textonempty-->";
                                } else {
                                    echo $textonempty;
                                }
                            }
                        }

                        ++$this->protection;
                        if ($this->protection > 25) {
                            error_log($this->protection . ' ' . $datacount . '-' . join(' ', $this->where));
                            return;
                        }
                        $this->datacount = null;
                        $this->dataitems = null;
                        $this->getDatacount();
                        break;

                    default:
                    case 'redirect':
                        $redirs = $this->basequery;
                        if ($f = $this->getInt('ajax', 0)) {
                            $redirs['ajax'] = $f;
                        }
                        $redirs['qt'] = $this->select['t'] ?? '';
                        $redirs['f'] = $this->select['f'] ?? '';

                        for ($v = $slug_level; $v >= 1; --$v) {
                            $redirs['q' . $v] = $this->select[$v] ?? '';
                        }
                        $redirs['format'] = $this->getVar('format', '');
                        $redirs['tmpl'] = $this->getVar('tmpl', '');
                        $redirect = self::makeRoute($redirs);

                        $app = Factory::getApplication();
                        if ($textonempty) {
                            $app->enqueueMessage($textonempty);
                        }
                        $app->redirect($redirect);
                        exit;
                }
            }
        }
        return;
    }

    public function compile_query()
    {
        $group_select = '';
        $where = [];
        $menu_filter = [];
        $this->where_level = 0;

        for ($v = DF_LEVELSTART; $v <= DF_LEVELEND; $v++) {
            $f = $this->params->get("field$v", "Empty");
            $s = $this->select[$v];
            if ('Empty' == $f) {
                if ($s) {
                    $this->select[$v] = '';
                }
            } else {
                $this->fields[$v] = $f;
                if ($s) {
                    $where[] = df_select($f, $s, false);
                    $this->where_level++;
                }
            }
        }

        $sorteer = $this->params->get('sorteer0', 'items_id');
        $volgorde = $this->params->get('volg0', 'DESC');




        $filter1 = $this->params->get('filter1', '');
        $filter1values = $this->params->get('filter1values', '');

        $filter2 = $this->params->get('filter2', '');
        $filter2values = $this->params->get('filter2values', '');


        if ($filter1 && $filter1values) {  //First filter
            $menu_filter[] = self::makeFilter($filter1, $filter1values);
        }
        if ($filter2 && $filter2values) { //Second filter
            $menu_filter[] = self::makeFilter($filter2, $filter2values);
        }


        if ($sqlfilter = $this->params->get('sqlfilter', '')) {
            $menu_filter[] = ' ( ' . $sqlfilter . ' ) ';
        }
        //don't group when price range selected
        if (
            !empty($this->select['min']) || !empty($this->select['max'])

        ) {
            $groepeer = false;
        } else {
            $groepeer = $this->params->get('groepeer', false);
        }
        $groep_alles = $this->params->get('groep_alles', false);


        if ($this->select['f']) {
            $where[] = df_select('Feed', $this->select['f'], false);
        }
        if ($this->select['t']) {
            $this->singleitem = true;
            $where[] = df_select('Title', $this->select['t'], false);
            $groepeer = false;
        }
        if ($this->select['id']) {
            $this->singleitem = true;
            $where[] = df_select('items_id', $this->select['id'], false);
            $groepeer = false;
        }

        $minprice = $this->select['min'] ??  $this->params->get('min_price', 0);
        $maxprice = $this->select['max'] ??  $this->params->get('max_price', 0);
        if (($minprice || $maxprice) &&
            ($pricequery = pricequery($minprice, $maxprice))
        ) {
            $menu_filter[] = $pricequery;
        }





        if ($this->where_level > 0) {
            $this->multipagina = $this->params->get('multi1', 0);
            $aantal = $this->params->get('aantal1', 20);
            //$sorttest=$this->params->get("sorteer".$this->where_level,$this->params->get("sorteer1",'default'));
            $sorttest = $this->params->get('sorteer1', 'default');
            if ('default' != $sorttest) {
                $volgorde = $this->params->get('volg1', 'DESC');
                $sorteer = $sorttest;
            }
        } else {
            $aantal = $this->params->get('aantal0', 20);
            $this->multipagina = $this->params->get('multi0', 0);
        }

        if (function_exists('custom_sort')) {
            if ($a = custom_sort($this->getWhere(true))) {
                $sorteer = $a['sort'];
                $volgorde = $a['order'];
            }
        }

        if ($groepeer) {
            if (('1' == $groepeer)) {
                $group_select = $this->fields[1];
            } else {
                $group_select = $groepeer;
            }
            if ($this->where_level > 0) {
                switch ((int) $groep_alles) {
                    case 0:
                        $group_select = '';
                        break;
                    case 1:
                        break;
                    default:

                        $v = $this->maxq + 1;
                        if ($v > $groep_alles) {
                            $group_select = '';
                        } else {
                            $group_select =   $this->fields[$v] ?? 'Empty';
                            $sorteer = ('Empty' == $group_select) ? $sorteer : $group_select;
                            $volgorde = ('Empty' == $group_select) ? $volgorde : 'ASC';
                        }
                }
                $group_select = 'Empty' == $group_select ? '' : $group_select;
            }
        }


        if (($s = $this->getVar('sort'))) {
            $v = StringHelper::strtolower($v);
            $sorteer = match ($s) {
                'h' => 'hits',
                'p', 'price' => 'price',
                'a', 'addate', 'items_id' => 'items_id',
                '0', '1', '2', '3', '4', '5', '6', '7', '8', '9' =>  $this->fields[$s] ?? 'items_id',
                'id', 'field0', 'field1', 'field2', 'field3', 'field4',
                'field5', 'field6', 'field7', 'field8', 'field9', 'field10',
                'field11', 'field12', 'field13', 'field14',
                'field15', 'field16', 'field17', 'field18', 'field19',
                'price', 'feed', 'title', 'hits' => $s,
                'rand()' => 'id',
                default => 'items_id'
            };
        };

        /*
	if ( $sorteer == 'id' ) {
         $this->_getFoundRows();
	 if ($this->datacount > $aantal) {
                $this->limitstart = rand(0, $this->datacount - $aantal);
            }
	}
	 */

        if (($v = $this->getVar('order', ''))) {
            $v = StringHelper::strtoupper($v);
            $volgorde = match ($v) {
                'ASC', 'A' => 'ASC',
                'DESC', 'D' => 'DESC',
                default => 'DESC'
            };
        };

        if ($this->select['q']) {
            if ($this->params->get('siblings', 'Empty') != $group_select) {
                $group_select = '';
            }
            $this->multipagina = true;
            $where[] = make_qQuery($this->select['q'], $this->db);
        } else {
            if ($this->where_level < 1 && !$this->singleitem) {
                $filterfirstpage = $this->params->get('filterfirstpage', '');
                $filterfirstpagevalues = $this->params->get('filterfirstpagevalues', '');
                if ($filterfirstpage and $filterfirstpagevalues) {
                    $where[] = self::makeFilter($filterfirstpage, $filterfirstpagevalues);
                }
                if ($sqlfilter_fp = trim($this->params->get('sqlfilter_fp', ''))) {
                    $where[] = ' ( ' . $sqlfilter_fp . ' ) ';
                }
            }
        }

        $this->sorteer = $sorteer;
        $this->volgorde = $volgorde;
        $this->aantal = $aantal;
        $this->group_select = $group_select;

        if (function_exists('custom_where')) {
            $menu_filter[] = custom_where($this->getWhere(true));
        }

        if ($menu_filter) {
            $where['menu_filter'] = join(' AND ', $menu_filter);
        }


        $this->where = $where ? $where : [1];
        return 1;
    }

    public function getSiblingfield()
    {
        return $this->params->get('siblings', 'Empty');
    }

    public function setSiblingGroup($a)
    {
        $this->params->set('siblinggroup', $a);
    }

    public function getSiblings($fields = [])
    {

        $siblings = [];
        if (null === $this->dataitems || 0 === count($this->dataitems)) {
            return $siblings;
        }

        $siblingfield = $this->params->get('siblings', '');
        $siblingcount = $this->params->get('siblingcount', 5);
        $siblinggroup = $this->params->get('siblinggroup', 'Empty');
        $siblingsort = $this->params->get('siblingsort', 'items_id');
        $siblingvolg = $this->params->get('siblingvolg', 'DESC');
        if ('Empty' === $siblingfield) {
            $siblingfield = '';
        }
        if (!$siblingfield) {
            return $siblings;
        }
        if (!('Empty' === $siblinggroup || '' === $siblinggroup)) {
            $group = $this->db->qn($siblinggroup);
        }



        foreach ($this->dataitems as $item) {
            $field = $item[$siblingfield] ?? '';
            if ($field && !isset($siblings[$field])) {
                $query = $this->db->createQuery()
                    ->select('*')
                    ->from($this->db->qn('#__dataitems'))
                    ->order($this->db->qn($siblingsort) . ' ' . $siblingvolg)
                    ->setLimit($siblingcount);

                if ($group) {
                    $query->group($group);
                }
                if ($this->where['menu_filter']) {
                    $query->where($this->where['menu_filter']);
                }
                $query->where($this->db->qn($siblingfield) . ' = ' . $this->db->q($field));
                foreach ($fields as $v) {
                    $query->where($this->db->qn($v) . ' = ' . $this->db->q($item[$v]));
                }
                $this->db->setQuery($query);
                $siblings[$field] = $this->db->loadAssocList();
            }
        }

        return $siblings;
    }

    public function getFeedinfo()
    {
        echo 'depricated';

        return false;
    }

    public function getDataitems()
    {
        if (null === $this->dataitems) {
            $this->_runQuery();
            $this->checkEmpty();
            $this->setPathway();
        }
        return $this->dataitems;
    }

    public function _runQuery()
    {

        $this->compile_query();
        $this->dataitems = [];

        $this->_getFoundRows();
        if (!$this->datacount) {
            return;
        }

        $order = $this->db->qn($this->sorteer) . ' ' . $this->volgorde;
        $query = $this->db->createQuery()
            ->select('*')
            ->from($this->db->qn('#__dataitems'))
            ->setLimit($this->aantal, $this->limitstart)
            ->where($this->where)
            ->order($order);

        if ($this->group_select) {
            $query->setLimit($this->aantal); //actually already limited by the inner query. No offset!
            $subquery = $this->db->createQuery()
                ->select($this->db->qn($this->group_select, 's1'))
                ->select(($this->volgorde == 'ASC' ? 'min' : 'max') . '(' . $this->db->qn($this->sorteer) . ') `s2`')
                ->from($this->db->qn('#__dataitems'))
                ->where($this->where)
                ->group($this->db->qn($this->group_select))
                ->order("`s2` {$this->volgorde},`items_id`")
                ->setLimit($this->aantal, $this->limitstart);

            // create an index on group_select and sorteer!
            $join1 = $this->db->qn($this->group_select) . ' = `sub`.`s1`';
            $join2 = $this->db->qn($this->sorteer) . ' = `sub`.`s2`';
            $query->innerJoin("($subquery) `sub` ON $join1 AND $join2")
                ->group($this->db->qn($this->group_select));
        }

        $this->db->setQuery($query);
        $this->dataitems = $this->db->loadAssocList();
    }

    public function _getFoundRows()
    {
        //avoid double queries
        if ($this->datacount  !== null) {
            return $this->datacount;
        }
        $query = $this->db->createQuery();
        $query
            ->from($this->db->qn('#__dataitems'))
            ->where($this->where);

        if ($this->group_select) {
            $query->select('count(DISTINCT ' . $this->db->qn($this->group_select) . ')');
        } else {
            $query->select('count(*)');
        }

        $this->db->setQuery($query);
        $this->datacount = $this->db->loadResult();
        return  $this->datacount;
    }

    public function getDatacount()
    {
        if (!$this->datacount) {
            $this->getDataitems();
        }

        return $this->datacount;
    }

    public function getBasequery()
    {
        return $this->basequery;
    }
    public function getSelect()
    {
        return $this->select;
    }
    public function getFields()
    {
        return $this->fields;
    }
    public function getWhere($basic = false)
    {
        $where = [];
        $slug = '';
        for ($v = DF_LEVELEND; $v >= DF_LEVELSTART; --$v) {
            if ($this->select[$v]) {
                if (is_array($this->select[$v])) {
                    foreach ($this->select[$v] as $value) {
                        $slug .= "&q$v" . '[]=' . urlencode($value);
                    }
                } else {
                    $slug .= "&q$v=" . urlencode($this->select[$v]);
                }
            }
        }
        $where['qt'] = $this->select['t'];
        $where['q'] = $this->select['q'];

        if ($this->select['q']) {
            $slug .= '&q=' . urlencode($this->select['q']);
        }
        $where['slug'] = $slug;
        $where['menu_id'] = $this->menu_id;
        $where['sort'] = $this->sorteer;
        if ($basic) {
            return $where;
        }
        $where['feed_path'] = $this->params->get('feed_path', '1,2,3');
        if ($this->limitstart > 0) {
            // in principe is limitstart altijd een veelvoud van aantal
            // behalve is de config gewijzigd is dus toch maar een floor
            $s = floor($this->limitstart / $this->aantal) + 1;
            $t = ceil($this->datacount / $this->aantal);
            if (defined('DF_PAGELIMIT')) {
                $t = min($t, DF_PAGELIMIT);
            }


            $this->title['page'] = sprintf(Text::_('JLIB_HTML_PAGE_CURRENT_OF_TOTAL'), $s, $t);
        }
        $where['title'] = $this->title;
        $where['aantal'] = $this->aantal;
        $where['level'] = $this->where_level; // the number of menu levels adressed in the query
        $where['next_level'] = $this->maxq + 1; // highest menu level address in the query plus one, so the first unqueried
        $where['chain'] = $this->chain; // highest menu level address in the query plus one, so the first unqueried
        $where['singleitem'] = $this->singleitem;
        $where['limitstart'] = $this->limitstart;
        if ($where['count'] = $this->getDatacount()) {
            $where['template'] = $this->where_level ? $this->params->get('template1', 'full') : $this->params->get('template', 'full');
        } else {
            $where['template'] = 'nothing';
        }

        return $where;
    }

    public function setPathway()
    {
        $pagelink = 'index.php?option=com_datafeeds&view=items&Itemid=' . $this->menu_id;
        $app = Factory::getApplication();
        $pathway = $app->getPathway();

        for ($v = DF_LEVELSTART; $v <= DF_LEVELEND; ++$v) {
            $q = $this->select[$v] ?? '';
            if ($q) {
                $pagelink .= "&q$v=" . urlencode($q);
                $pathway->addItem($this->title[$v], $pagelink);
            }
        }
        $q = $this->select['q'] ?? '';
        if ($q) {
            $pagelink = 'index.php?option=com_datafeeds&view=items&Itemid=' . $this->menu_id . '&q=' . urlencode($q);
            $pathway->addItem($q, $pagelink);
        }
        $q = $this->select['t'] ?? '';
        if ($q) {
            $pagelink = 'index.php?option=com_datafeeds&view=items&Itemid=' . $this->menu_id . '&qt=' . urlencode($q);
            $pathway->addItem($q, $pagelink);
        }
        $q = $this->select['f'] ?? '';
        if ($q) {
            $pagelink = 'index.php?option=com_datafeeds&view=items&Itemid=' . $this->menu_id . '&qf=' . urlencode($q);
            $pathway->addItem($q, $pagelink);
        }
    }

    public function getPagination()
    {
        if ((($count = $this->getDatacount()) > $this->aantal) &&
            ($this->multipagina) && ('component' != $this->getVar('tmpl'))
        ) {
            if (empty($this->pagination)) {
                if (defined('DF_PAGELIMIT')) {
                    $count = min($count, DF_PAGELIMIT * $this->aantal);
                }
                $this->pagination = new Pagination($count, $this->limitstart, $this->aantal);
            }
        } else {
            $this->pagination = null;
        }

        return $this->pagination;
    }

    public function getNext($id = 0)
    {

        $query = $this->db->createQuery()
            ->select('*')
            ->from($this->db->qn('#__dataitems'))
            ->where('`items_id` > ' . (int) $id)
            ->setLimit(1);
        if ($this->where['menu_filter']) {
            $query->where($this->where['menu_filter']);
        }

        $this->db->setQuery($query);
        $next = $this->db->loadAssoc();
        return $next;
    }

    public function getPrevious($id = 0)
    {

        $query = $this->db->createQuery()
            ->select('*')
            ->from($this->db->qn('#__dataitems'))

            ->where('`items_id` < ' . (int) $id)
            ->setLimit(1);

        if ($this->where['menu_filter']) {
            $query->where($this->where['menu_filter']);
        }

        $this->db->setQuery($query);
        $previous = $this->db->loadAssoc();
        return $previous;
    }

    public function getRelated($q = '', $limit = 10, $order = 'hits DESC')
    {

        $query = $this->db->createQuery()
            ->select('*')
            ->from($this->db->qn('#__dataitems'))
            ->setLimit($limit);

        if ($q) { // only developers will add/call this function they must check
            $query->where("( {$q} )");
        }
        if ($order) { // only developers will add/call this function they must check
            $query->order($order);
        }

        if ($this->where['menu_filter']) {
            $query->where($this->where['menu_filter']);
        }

        $this->db->setQuery($query);
        $related = $this->db->loadAssocList();
        return $related;
    }

    public static  function replaceRoute(array $params, array $replace)
    {
        foreach ($replace as $q => $i) {
            if ($i) {
                $params[$q] = $i;
            } else {
                unset($params[$q]);
            }
        }

        return $params;
    }
    public static  function makeRoute(array $params, string|bool|array $q = false, string|bool $i = false)
    {
        if ($q) {
            if (is_array($q)) {
                $params = self::replaceRoute($params, $q);
            } else {
                $params = self::replaceRoute($params, [$q => $i]);
            }
        }
        ## Like in  AbstractUri.php saving the  Uri::getInstance()->setQuery.....
        ## Maybe the urldecode is not needed here. In Route::_ the whole lot goes thru encoding again
        $query = urldecode(http_build_query($params, '', '&'));
        return Route::_('index.php?' . $query, false);
    }
    public static function makeFilter($field, $group)
    {
        $db = Factory::getContainer()->get('DatabaseDriver');
        $groep_array = [];
        $not_groep_array = [];
        if ('Empty' == $field) {
            return 1;
        }
        foreach (explode(',', $group) as $g) {
            if (strlen($g)) {
                if ('-' === $g[0]) {
                    $g = substr($g, 1);
                    $not_groep_array[] = $db->Quote($g);
                } else {
                    $groep_array[] = $db->Quote($g);
                }
            }
        }
        if (count($groep_array)) {
            $groep = join(',', $groep_array);
            return $db->qn($field) . " IN  ($groep)";
        }

        if (count($not_groep_array)) {
            $not_groep = join(',', $not_groep_array);
            return $db->qn($field) . "  NOT IN ($not_groep)";
        }
        return 1;
    }

    static  function fixParamNames(Registry $f): Registry
    {
        foreach ([
                'field0' => 'filter1',
                'groep' => 'filter1values',
                'field0_2' => 'filter2',
                'groep_2' => 'filter2values',
                'field0_fp' => 'filterfirstpage',
                'groep_fp' => 'filterfirstpagevalues',
                'xfield0' => 'field0',



            ] as $from => $to) {
            $f->set($to, $f->get($from, ''));
        }


        return $f;
    }
    /* moneyFormat is a small helper function to print number strings based on the locale.
 * you could use number_format directly in your template overrides
 * the php function money_format is depricated
 */
    static  function moneyFormat(float $f): string
    {
        if (class_exists('NumberFormatter') && defined('DF_USE_NUMBERFORMATTER')) {
            $locale = Factory::getLanguage()->getLocale()[0];
            $fmt = new \NumberFormatter($locale, \NumberFormatter::DECIMAL);
            $fmt->setAttribute(\NumberFormatter::FRACTION_DIGITS, 2);
            return $fmt->format($f);
        } else {
            $locale_info = localeconv();
            #Since 2003, the Internationa Standard of Units regulated that groups of three are to be separated by spaces and not dots or commas:"
            return number_format($f, 2, $locale_info['mon_decimal_point'] ?? '.', ' ');
        }
    }
}
