<?php


/**
 * @version 23.11.dev
 * @package    Datafeeds for Joomla
 * @author     Bram <bram@brambring.nl>
 * @copyright  2023 Bram Brambring
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 */

\defined('_JEXEC') or die('Restricted access');


use Joomla\CMS\Factory;
use Joomla\CMS\Uri\Uri;

use Joomla\String\StringHelper;


$base = Uri::base(false);
$input = Factory::getApplication()->input;
$slogan = $input->get('slogan', '', 'string');
$source = $input->get('source', 'EMBED', 'string');
$img_x = 200;
$img_y = floor(1.5 * $img_x);
$id = uniqid();

header('Expires: ' . gmdate('D, d M Y H:i:s \G\M\T', time() + (60 * 60)));
header("X-Robots-Tag: noindex");

if ($slogan) {
    print '<h2>' . $slogan . '</h2>';
}

echo '<div class="q-' . $id . '">';

foreach ($this->items as $item) {
    # deze hbben altijd een waarde. Geen check nodig
    $title = StringHelper::ucwords(StringHelper::strtolower($item['title']));
    $image = $item['image'];
    $price = (float)$item['price']; //decimal in database returns as string
    $items_id = $item['items_id'];

    $ref  = $base . 'index.php?option=com_datafeeds&task=go&format=raw&item=' . $items_id;
    $ref .= '&p=' . urlencode($source);

    $extra = [];
    foreach (['field1', 'field2', 'field3', 'feed'] as $field) {
        $f = $item[$field] ?? '';
        if ($f) {
            $extra[] = $f;
        }
    }


    if ($price > 0) {
        $pformated = number_format($price, 2, ",", " ");
        $price = '<span class="price">€ ' . $pformated . '</span>';
    }
    echo '<figure><a target="_blank" rel="nofollow" href="' . $ref . '"><img loading="lazy"  src="' . $image . '" width="' . $img_x . '"   alt="" ><figcaption><p><strong>' . $title . '</strong>' . $price . join(' - ', $extra) . '</figcaption></a></figure>';
}
echo '</div>';

?>
<style>
    div.q-<?php echo $id; ?> {
        display: flex;
        flex-wrap: wrap;
        justify-content: space-around;
    }

    div.q-<?php echo $id; ?>figure {
        flex: 0 0 auto;
    }

    @media (max-width: <?php echo $img_x * 6 + 10; ?>px) {
        div.q-<?php echo $id; ?>figure:nth-child(n+5) {
            display: none;
        }
    }

    @media (max-width: <?php echo $img_x * 3 + 10; ?>px) {
        div.q-<?php echo $id; ?>figure:nth-child(n+3) {
            display: none;
        }

        div.q-<?php echo $id; ?>figure {
            border: none;
            margin: 1px 1px 5px 1px;
        }
    }

    div.q-<?php echo $id; ?>figure {
        margin: 10px 5px;
        width: <?php echo $img_x; ?>px;
        background-color: #eee;
        color: #fff;
        box-shadow: 1px 1px 2px 0 grey;
        height: auto;
    }

    div.q-<?php echo $id; ?>figure:hover {
        box-shadow: 1px 1px 6px 0 grey;
    }

    div.q-<?php echo $id; ?>img {
        width: <?php echo $img_x; ?>px;
        height: <?php echo $img_y; ?>px;
        object-fit: contain;
    }

    div.q-<?php echo $id; ?>figcaption {
        padding: 5px 1px 1px 1px;
        margin: 0px;
        word-wrap: break-word;
        bottom: 0;
        text-align: left;
        color: #222 !important;
    }

    div.q-<?php echo $id; ?>strong {
        color: #1f99c4;
        display: block;
        font-size: 14px;
        height: 2.2em;
        overflow: hidden;
        position: relative;
        text-overflow: ellipsis;
        font-weight: 300;
        line-height: 1.1em;
    }

    div.q-<?php echo $id; ?>p {
        margin: 3px;
        font-size: 13px;
    }

    div.q-<?php echo $id; ?>span.price {
        font-weight: 700;
        white-space: nowrap;
        display: block;
    }

    div.q-<?php echo $id; ?>a {
        text-decoration: none;
    }
</style>